PLOTLY_THEME = 'plotly_dark'
REPORT_TOKEN = '@@'
MODEL_TOKEN = '::'
DATASET_TOKEN = ', '
DEFAULT_BAR_WIDTH = 0.2
LATEX_DELIMITERS = [{
    'left': '$$',
    'right': '$$',
    'display': True
}, {
    'left': '$',
    'right': '$',
    'display': False
}, {
    'left': '\\(',
    'right': '\\)',
    'display': False
}, {
    'left': '\\[',
    'right': '\\]',
    'display': True
}]
