# flake8: noqa
language_dic = {
    'en': 'English',
    'zh': 'Chinese',
    'ar': 'Arabic',
    'bn': 'Bangali',
    'de': 'German',
    'es': 'Spanish',
    'fr': 'French',
    'id': 'Indonesian',
    'it': 'Italian',
    'ja': 'Japanese',
    'ko': 'Korean',
    'ms': 'Malay',
    'pt': 'Potuguese',
    'ru': 'Russian',
    'sw': 'Swahili',
    'te': 'Telugu',
    'th': 'Thai',
    'vi': 'Vietnamese',
}

query_dic = {
    'en': 'Note: Please put the final answer in the $\\boxed\{\}$.',
    'zh': '注意：请将最终答案放在 $\\boxed\{\}$ 中。',
    'ar': 'ملاحظة: يُرجى وضع الإجابة النهائية في $\\boxed\{\}$.',
    'bn': 'বিঃদ্রঃ: অনুগ্রহ করে চূড়ান্ত উত্তরটি $\\boxed\{\}$ এর মধ্যে রাখুন।',
    'de': 'Hinweis: Bitte setzen Sie die endgültige Antwort in $\\boxed\{\}$.',
    'es': 'Nota: Por favor, coloque la respuesta final en el $\\boxed\{\}$.',
    'fr': 'Remarque : Veuillez mettre la réponse finale dans le $\\boxed\{\}$.',
    'id': 'Catatan: Silakan letakkan jawaban akhir di dalam $\\boxed\{\}$.',
    'it': 'Nota: Per favore, metti la risposta finale nel $\\boxed\{\}$.',
    'ja': '注意：最終的な答えを $\\boxed\{\}$ に入れてください。',
    'ko': '참고: 최종 답안을 $\\boxed\{\}$ 안에 넣어 주세요.',
    'ms': 'Nota: Sila letakkan jawapan akhir dalam $\\boxed\{\}$.',
    'pt': 'Nota: Por favor, coloque a resposta final no $\\boxed\{\}$.',
    'ru': 'Примечание: Пожалуйста, поместите окончательный ответ в $\\boxed\{\}$.',
    'sw': 'Kumbuka: Tafadhali weka jibu la mwisho katika $\\boxed\{\}$.',
    'te': 'గమనిక: దయచేసి తుది జవాబును $\\boxed\{\}$ లో ఉంచండి.',
    'th': 'หมายเหตุ: กรุณาใส่คำตอบสุดท้ายใน $\\boxed\{\}$.',
    'vi': 'Lưu ý: Vui lòng đặt câu trả lời cuối cùng trong $\\boxed\{\}$.',
}

language_control = {
    'forcing_raw': {
        'en': 'Use English to think and answer.',
        'zh': '使用中文进行思考和回答。',
        'ar': 'استخدم العربية للتفكير والإجابة.',
        'bn': 'বাংলা ব্যবহার করে চিন্তা এবং উত্তর দিন।',
        'de': 'Verwende Deutsch, um zu denken und zu antworten.',
        'es': 'Usa español para pensar y responder.',
        'fr': 'Utilisez le français pour penser et répondre.',
        'id': 'Gunakan bahasa Indonesia untuk berpikir dan menjawab.',
        'it': 'Usa italiano per pensare e rispondere.',
        'ja': '日本語を使って考え、回答してください。',
        'ko': '한국어로 생각하고 답변하세요.',
        'ms': 'Gunakan bahasa Melayu untuk berfikir dan menjawab.',
        'pt': 'Use português para pensar e responder.',
        'ru': 'Используйте русский язык для размышлений и ответов.',
        'sw': 'Tumia Kiswahili kufikiri na kujibu.',
        'te': 'తెలుగును ఉపయోగించి ఆలోచించి సమాధానం ఇవ్వండి.',
        'th': 'ใช้ภาษาไทยในการคิดและตอบคำถาม.',
        'vi': 'Sử dụng tiếng Việt để suy nghĩ và trả lời.',
    },
    'forcing_en': {
        'en': 'Use English to think and answer.',
        'zh': '使用英文进行思考和回答。',
        'ar': 'استخدم اللغة الإنجليزية للتفكير والإجابة.',
        'bn': 'ইংরেজি ব্যবহার করে চিন্তা এবং উত্তর দিন।',
        'de': 'Verwenden Sie Englisch, um zu denken und zu antworten.',
        'es': 'Usa inglés para pensar y responder.',
        'fr': "Utilisez l'anglais pour penser et répondre.",
        'id': 'Gunakan bahasa Inggris untuk berpikir dan menjawab.',
        'it': 'Usa inglese per pensare e rispondere.',
        'ja': '英語を使って考え、回答してください。',
        'ko': '영어로 생각하고 답변하세요.',
        'ms': 'Gunakan bahasa Inggeris untuk berfikir dan menjawab.',
        'pt': 'Use inglês para pensar e responder.',
        'ru': 'Используйте английский язык, чтобы думать и отвечать.',
        'sw': 'Tumia Kiingereza kufikiri na kujibu.',
        'te': 'ఇంగ్లీష్‌ను ఉపయోగించి ఆలోచించి ఉత్తరించండి.',
        'th': 'ใช้ภาษาอังกฤษในการคิดและตอบคำถาม.',
        'vi': 'Sử dụng tiếng Anh để suy nghĩ và trả lời.',
    },
    'forcing_prefer': {
        'en': 'Choose the language you are most proficient in to think and answer.',
        'zh': '自选一种你最擅长的语言进行思考和回答。',
        'ar': 'اختر اللغة التي تجيدها أكثر للتفكير والإجابة.',
        'bn': 'আপনি যে ভাষাটি সবচেয়ে পারদর্শী সেটি বেছে নিয়ে চিন্তা এবং উত্তর দিন।',
        'de': 'Wählen Sie die Sprache, in der Sie am kompetentesten sind, um zu denken und zu antworten.',
        'es': 'Elige el idioma en el que eres más competente para pensar y responder.',
        'fr': 'Choisissez la langue dans laquelle vous êtes le plus compétent pour penser et répondre.',
        'id': 'Pilih bahasa yang paling Anda kuasai untuk berpikir dan menjawab.',
        'it': 'Scegli la lingua in cui sei più competente per pensare e rispondere.',
        'ja': '最も得意な言語を選んで考え、回答してください。',
        'ko': '가장 능숙한 언어를 선택하여 생각하고 답변하세요.',
        'ms': 'Pilih bahasa yang paling anda mahir untuk berfikir dan menjawab.',
        'pt': 'Escolha o idioma em que você é mais competente para pensar e responder.',
        'ru': 'Выберите язык, в котором вы наиболее компетентны, чтобы думать и отвечать.',
        'sw': 'Chagua lugha ambayo unamudu zaidi kufikiri na kujibu.',
        'te': 'మీరు అత్యంత స్థిరంగా ఉన్న భాషను స్వీకరించి ఆలోచించిตอบ。',
        'th': 'เลือกภาษาที่คุณมีความสามารถมากที่สุดในการคิดและตอบคำถาม.',
        'vi': 'Chọn ngôn ngữ mà bạn thành thạo nhất để suy nghĩ và trả lời.',
    }
}
